﻿; MinMaxWindow.ahk[UTF-8] (2024/02/10 13:00)
; for AutoHotKey v2.0.12.0 or later
;
; 指定のウィンドウを最小化や最大化する
;    MinMaxWindow Mode Param (Activate) (Post) (Debug)
;       Mode 
;          0/min/minimize : 最小化する
;          1/max/maximize : 最大化する
;          2/res/restore  : 元に戻す
;       Param
;          ウィンドウ文字列/ahk_exe フルパス/ahk_class クラスID など両端""でAutoHotKey
;          独自のウィンドウ指定記述、現在アクティブなウィンドウを指定する場合は"A"
;          https://ahkscript.github.io/ja/docs/v2/misc/WinTitle.htm
;          対象ウィンドウのウィンドウスタイルは WS_VISIBLE ビットが有効で WS_DISABLED ビットが無効の
;          ものとしています。複数ウィンドウに該当する場合はエラー表示し処理しません。
;          ex) "ahk_exe C:\Program Files\Mozilla Firefox\firefox.exe"
;          ex) "ahk_exe C:\Program Files\Vivaldi\Application\vivaldi.exe ahk_class Chrome_WidgetWin_1"
;       Activate
;          1/on : ウィンドウをアクティブにする(別の仮想デスクトップでは動作しない)
;          [any] : 上記以外が指定されるか無指定ならアクティブにはしない
;       Post
;          1/on : PostMessageによる制御で切り替える
;          [any] : 上記以外が指定されるか無指定ならAutoHotKeyの関数にて処理する
;       Debug
;          [any] : MsgBoxでクラス名とウィンドウスタイルを表示、無指定なら表示なし
;

/* 同時に複数実行を許可する */
#SingleInstance Off

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておかないと仮想デスクトップで動作しない */
DetectHiddenWindows True

WM_SYSCOMMAND := 0x112
SC_MINIMIZE := 0xF020
SC_MAXIMIZE := 0xF030
SC_RESTORE := 0xF120
WS_DISABLED := 0x8000000
WS_VISIBLE := 0x10000000

/* コマンドオプション数チェック */
if( A_Args.Length < 2 ){
	DispUsage()
}

/* モードとウィンドウ指定 */
iMode := A_Args[1]
strParam := A_Args[2]
iAct := 0
iPost := 0
UniqueID := 0

if( StrUpper(iMode) = "MIN" or StrUpper(iMode) = "MINIMIZE" ){
	iMode := 0
}
if( StrUpper(iMode) = "MAX" or StrUpper(iMode) = "MAXIMIZE" ){
	iMode := 1
}
if( StrUpper(iMode) = "RES" or StrUpper(iMode) = "RESTORE" ){
	iMode := 2
}

/* フルパスに合致するウィンドウを列挙(非表示ウィンドウも対象としているため沢山ヒットする) */
listWin := WinGetList(strParam)
idxWin := 0
findCount := 0

/* ウィンドウスタイルでマスク */
for hWin in listWin{
	idxWin := idxWin + 1
	style := WinGetStyle("ahk_id " hWin)
	if( style & WS_VISIBLE and not(style & WS_DISABLED) ){
		findCount := findCount + 1
		UniqueID := hWin
		if( A_Args.Length >= 5 ){
			MsgBox("Class[" idxWin "]=" WinGetClass("ahk_id " hWin) " / style=0x" StrUpper(Format("{1:X}",style)) )
		}
	}
}

;UniqueID := WinExist(strParam)
if( UniqueID = 0 ){
	MsgBox(strParam " not found.")
	ExitApp()
}

if( findCount != 1 ){
	MsgBox(strParam " has multiple windows.")
	ExitApp()
}

/* WinActivateを実施する場合 */
if( A_Args.Length >= 3 ){
	iAct := A_Args[3]
}
if( iAct = 1 or StrUpper(iAct) = "ON" ){
	try{
		WinActivate("ahk_id " UniqueID)
	}catch TargetError as err{
		MsgBox(strParam " can not activate")
		ExitApp()
	}
}

/* PostMessageによる制御かどうか */
if( A_Args.Length >= 4 ){
	iPost := A_Args[4]
}
if( iPost = 1 or StrUpper(iPost) = "ON" ){
	iPost := 1
}

if( iPost = 0 ){
	if( iMode = 0 ){
		WinMinimize("ahk_id " UniqueID)
	} else if( iMode = 1 ){
		WinMaximize("ahk_id " UniqueID)
	} else if( iMode = 2 ){
		WinRestore("ahk_id " UniqueID)
	} else {
		DispUsage()
	}
} else {
	if( iMode = 0 ){
		PostMessage( WM_SYSCOMMAND, SC_MINIMIZE,,, "ahk_id " UniqueID)
	} else if( iMode = 1 ){
		PostMessage( WM_SYSCOMMAND, SC_MAXIMIZE,,, "ahk_id " UniqueID)
	} else if( iMode = 2 ){
		PostMessage( WM_SYSCOMMAND, SC_RESTORE,,, "ahk_id " UniqueID)
	} else {
		DispUsage()
	}
}

ExitApp()

DispUsage(){
	msg := 'Usage:`n'
	msg := msg 'MinMaxWindow.exe Mode Param (Activate) (Post)`n`n'
	msg := msg '[Mode]`n'
	msg := msg '    0/min/minimize = minimize mode,`n'
	msg := msg '    1/max/maximize = maxmize mode,`n'
	msg := msg '    2/res/restore = restore mode`n'
	msg := msg '[Param]`n'
	msg := msg '   "window-title`n'
	msg := msg '    ahk_exe full-path`n'
	msg := msg '    ahk_class class-name..."`n'
	msg := msg '[Activate]`n'
	msg := msg '    1/on = with WinActivate`n'
	msg := msg '[Post]`n'
	msg := msg '    1/on = use PostMessage'
	MsgBox(msg)
	ExitApp()
}


